@@ -798,13 +798,14 @@ int lo_server_recv(lo_server s);
  * matching message is received
  * \param user_data A value that will be passed to the callback function, h,
  * when its invoked matching from this method.
+ * \return A unique pointer identifying the method.  It should not be freed.
  */
 lo_method lo_server_add_method(lo_server s, const char *path,
                                const char *typespec, lo_method_handler h,
-                               void *user_data);
+                               const void *user_data);
 
 /**
- * \brief Delete an OSC method from the specifed server.
+ * \brief Delete an OSC method from the specified server.
  *
  * \param s The server the method is to be removed from.
  * \param path The OSC path of the method to delete. If NULL is passed the
@@ -814,6 +815,16 @@ lo_method lo_server_add_method(lo_server s, const char *path,
 void lo_server_del_method(lo_server s, const char *path,
                                const char *typespec);
 
+/**
+ * \brief Delete a specific OSC method from the specified server.
+ *
+ * \param s The server the method is to be removed from.
+ * \param m The lo_method identifier returned from lo_server_add_method for
+ *          the method to delete from the server.
+ * \return Non-zero if it was not found in the list of methods for the server.
+ */
+int lo_server_del_lo_method(lo_server s, lo_method m);
+
 /**
  * \brief Add bundle notification handlers to the specified server.
  *
@@ -899,6 +910,26 @@ int lo_server_events_pending(lo_server s);
  */
 double lo_server_next_event_delay(lo_server s);
 
+/** 
+ * \brief Set the maximum message size accepted by a server.
+ *
+ * For UDP servers, the maximum message size cannot exceed 64k, due to
+ * the UDP transport specifications.  For TCP servers, this number may
+ * be larger, but be aware that one or more contiguous blocks of
+ * memory of this size may be allocated by liblo.  (At least one per
+ * connection.)
+ *
+ * \param s The server on which to operate.
+ * \param req_size The new maximum message size to request, 0 if it
+ * should not be modified, or -1 if it should be set to unlimited.
+ * Note that an unlimited message buffer may make your application
+ * open to a denial of service attack.
+ * \return The new maximum message size is returned, which may or may
+ * not be equal to req_size.  If -1 is returned, maximum size is
+ * unlimited.
+ */
+int lo_server_max_msg_size(lo_server s, int req_size);
+
 /**
  * \brief Return the protocol portion of an OSC URL, eg. udp, tcp.
  *
