@@ -41,6 +41,9 @@
 #include <winsock2.h>
 #include <ws2tcpip.h>
 #include <malloc.h>
+// TODO: Does this make sense on any platform? At least mingw defines
+// both EADDRINUSE and WSAEADDRINUSE and the values differ. We need to
+// check for WSAEADDRINUSE though on Windows.
 #ifndef EADDRINUSE
 #define EADDRINUSE WSAEADDRINUSE
 #endif
@@ -78,6 +81,7 @@ typedef struct {
 } queued_msg_list;
 
 struct lo_cs lo_client_sockets = { -1, -1 };
+static int reuseport_supported = 1;
 
 static int lo_can_coerce_spec(const char *a, const char *b);
 static int lo_can_coerce(char a, char b);
@@ -100,7 +104,7 @@ static int lo_server_join_multicast_group(lo_server s, const char *group,
 
 #if defined(WIN32) || defined(_MSC_VER)
 #ifndef gai_strerror
-// Copied from the Win32 SDK 
+// Copied from the Win32 SDK
 
 // WARNING: The gai_strerror inline functions below use static buffers,
 // and hence are not thread-safe.  We'll use buffers long enough to hold
@@ -166,8 +170,8 @@ static int detect_windows_server_2003_or_later()
     VER_SET_CONDITION( dwlConditionMask, VER_MINORVERSION, op );
 
     return VerifyVersionInfo(
-        &osvi, 
-        VER_MAJORVERSION | VER_MINORVERSION | 
+        &osvi,
+        VER_MAJORVERSION | VER_MINORVERSION |
         VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
         dwlConditionMask);
 }
@@ -189,26 +193,28 @@ static unsigned int get_family_PF(const char *ip, const char *port)
 }
 #endif
 
-static int lo_server_setsock_reuseaddr(lo_server s)
+static int lo_server_setsock_reuseaddr(lo_server s, int do_throw)
 {
     unsigned int yes = 1;
     if (setsockopt(s->sockets[0].fd, SOL_SOCKET, SO_REUSEADDR,
                    (char*)&yes, sizeof(yes)) < 0) {
         int err = geterror();
-        lo_throw(s, err, strerror(err), "setsockopt(SO_REUSEADDR)");
+		if (do_throw)
+			lo_throw(s, err, strerror(err), "setsockopt(SO_REUSEADDR)");
         return err;
     }
     return 0;
 }
 
-static int lo_server_setsock_reuseport(lo_server s)
+static int lo_server_setsock_reuseport(lo_server s, int do_throw)
 {
 #ifdef SO_REUSEPORT
     unsigned int yes = 1;
     if (setsockopt(s->sockets[0].fd, SOL_SOCKET, SO_REUSEPORT,
                    &yes, sizeof(yes)) < 0) {
         int err = geterror();
-        lo_throw(s, err, strerror(err), "setsockopt(SO_REUSEPORT)");
+		if (do_throw)
+			lo_throw(s, err, strerror(err), "setsockopt(SO_REUSEPORT)");
         return err;
     }
 #endif
@@ -284,6 +290,49 @@ lo_server lo_server_new_from_url(const char *url,
     return s;
 }
 
+static
+void lo_server_resolve_hostname(lo_server s)
+{
+    char hostname[LO_HOST_SIZE];
+
+    /* Set hostname to empty string */
+    hostname[0] = '\0';
+
+#ifdef ENABLE_IPV6
+    /* Try it the IPV6 friendly way first */
+    for (it = ai; it; it = it->ai_next) {
+        if (getnameinfo(it->ai_addr, it->ai_addrlen, hostname,
+                        sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
+            break;
+        }
+    }
+
+    /* check to make sure getnameinfo() didn't just set the hostname to "::".
+       Needed on Darwin. */
+    if (hostname[0] == ':') {
+        hostname[0] = '\0';
+    }
+#endif
+
+    /* Fallback to the oldschool (i.e. more reliable) way */
+    if (!hostname[0]) {
+        struct hostent *he;
+
+        gethostname(hostname, sizeof(hostname));
+        he = gethostbyname(hostname);
+        if (he) {
+            strncpy(hostname, he->h_name, sizeof(hostname));
+        }
+    }
+
+    /* somethings gone really wrong, just hope its local only */
+    if (!hostname[0]) {
+        strcpy(hostname, "localhost");
+    }
+
+    s->hostname = strdup(hostname);
+}
+
 lo_server lo_server_new_with_proto_internal(const char *group,
                                             const char *port,
                                             const char *iface,
@@ -297,7 +346,6 @@ lo_server lo_server_new_with_proto_internal(const char *group,
     int tries = 0;
     char pnum[16];
     const char *service;
-    char hostname[LO_HOST_SIZE];
     int err = 0;
 
 #if defined(WIN32) || defined(_MSC_VER)
@@ -321,7 +369,7 @@ lo_server lo_server_new_with_proto_internal(const char *group,
         return NULL;
 #endif
 
-    s = calloc(1, sizeof(struct _lo_server));
+    s = (lo_server) calloc(1, sizeof(struct _lo_server));
     if (!s)
         return 0;
 
@@ -330,14 +378,14 @@ lo_server lo_server_new_with_proto_internal(const char *group,
     s->ai = NULL;
     s->hostname = NULL;
     s->protocol = proto;
-    s->flags = LO_SERVER_DEFAULT_FLAGS;
+    s->flags = (lo_server_flags) LO_SERVER_DEFAULT_FLAGS;
     s->port = 0;
     s->path = NULL;
     s->queued = NULL;
     s->sockets_len = 1;
     s->sockets_alloc = 2;
-    s->sockets = calloc(2, sizeof(*(s->sockets)));
-    s->contexts = calloc(2, sizeof(*(s->contexts)));
+    s->sockets = (lo_server_fd_type *) calloc(2, sizeof(*(s->sockets)));
+    s->contexts = (struct socket_context *) calloc(2, sizeof(*(s->contexts)));
     s->sources = (lo_address)calloc(2, sizeof(struct _lo_address));
     s->sources_len = 2;
     s->bundle_start_handler = NULL;
@@ -355,6 +403,7 @@ lo_server lo_server_new_with_proto_internal(const char *group,
     }
 
     s->sockets[0].fd = -1;
+    s->max_msg_size = LO_DEFAULT_MAX_MSG_SIZE;
 
     memset(&hints, 0, sizeof(hints));
 
@@ -485,17 +534,18 @@ lo_server lo_server_new_with_proto_internal(const char *group,
         if (group != NULL
             || proto == LO_TCP)
         {
-            err = lo_server_setsock_reuseaddr(s);
+            err = lo_server_setsock_reuseaddr(s, 1);
             if (err) {
                 lo_server_free(s);
                 return NULL;
             }
         }
 
-        if (group != NULL)
+        if (group != NULL && reuseport_supported)
         {
             /* Ignore the error if SO_REUSEPORT wasn't successful. */
-            lo_server_setsock_reuseport(s);
+            if (lo_server_setsock_reuseport(s, 0))
+				reuseport_supported = 0;
         }
 
 #if defined(WIN32) || defined(_MSC_VER)
@@ -511,7 +561,11 @@ lo_server lo_server_new_with_proto_internal(const char *group,
             (bind(s->sockets[0].fd, used->ai_addr, used->ai_addrlen) <
              0)) {
             err = geterror();
+#ifdef WIN32
+            if (err == EINVAL || err == WSAEADDRINUSE) {
+#else
             if (err == EINVAL || err == EADDRINUSE) {
+#endif
                 used = NULL;
                 continue;
             }
@@ -549,43 +603,6 @@ lo_server lo_server_new_with_proto_internal(const char *group,
         lo_client_sockets.tcp = s->sockets[0].fd;
     }
 
-    /* Set hostname to empty string */
-    hostname[0] = '\0';
-
-#ifdef ENABLE_IPV6
-    /* Try it the IPV6 friendly way first */
-    for (it = ai; it; it = it->ai_next) {
-        if (getnameinfo(it->ai_addr, it->ai_addrlen, hostname,
-                        sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
-            break;
-        }
-    }
-
-    /* check to make sure getnameinfo() didn't just set the hostname to "::".
-       Needed on Darwin. */
-    if (hostname[0] == ':') {
-        hostname[0] = '\0';
-    }
-#endif
-
-
-    /* Fallback to the oldschool (i.e. more reliable) way */
-    if (!hostname[0]) {
-        struct hostent *he;
-
-        gethostname(hostname, sizeof(hostname));
-        he = gethostbyname(hostname);
-        if (he) {
-            strncpy(hostname, he->h_name, sizeof(hostname));
-        }
-    }
-
-    /* soethings gone really wrong, just hope its local only */
-    if (!hostname[0]) {
-        strcpy(hostname, "localhost");
-    }
-    s->hostname = strdup(hostname);
-
     if (used->ai_family == PF_INET6) {
         struct sockaddr_in6 *addr = (struct sockaddr_in6 *) used->ai_addr;
 
@@ -690,8 +707,9 @@ int lo_server_join_multicast_group(lo_server s, const char *group,
 int lo_server_enable_coercion(lo_server s, int enable)
 {
     int r = (s->flags & LO_SERVER_COERCE) != 0;
-    s->flags = (s->flags & ~LO_SERVER_COERCE)
-        | (enable ? LO_SERVER_COERCE : 0);
+    s->flags = (lo_server_flags)
+        ((s->flags & ~LO_SERVER_COERCE)
+         | (enable ? LO_SERVER_COERCE : 0));
     return r;
 }
 
@@ -736,7 +754,7 @@ void lo_server_free(lo_server s)
             s->path = NULL;
         }
         while (s->queued) {
-            queued_msg_list *q = s->queued;
+            queued_msg_list *q = (queued_msg_list *) s->queued;
             free(q->path);
             lo_message_free(q->msg);
             s->queued = q->next;
@@ -744,8 +762,8 @@ void lo_server_free(lo_server s)
         }
         for (it = s->first; it; it = next) {
             next = it->next;
-            free((char *) it->path);
-            free((char *) it->typespec);
+            free((char*) it->path);
+            free((char*) it->typespec);
             free(it);
         }
         if (s->addr_if.iface)
@@ -778,8 +796,9 @@ int lo_server_enable_queue(lo_server s, int enable,
                            int dispatch_remaining)
 {
     int prev = (s->flags & LO_SERVER_ENQUEUE) != 0;
-    s->flags = (s->flags & ~LO_SERVER_ENQUEUE)
-        | (enable ? LO_SERVER_ENQUEUE : 0);
+    s->flags = (lo_server_flags)
+        ((s->flags & ~LO_SERVER_ENQUEUE)
+         | (enable ? LO_SERVER_ENQUEUE : 0));
 
     if (!enable && dispatch_remaining && s->queued)
         dispatch_queued(s, 1);
@@ -789,20 +808,39 @@ int lo_server_enable_queue(lo_server s, int enable,
 
 void *lo_server_recv_raw(lo_server s, size_t * size)
 {
-    char buffer[LO_MAX_MSG_SIZE];
-    int ret;
+    char *buffer = NULL;
+    int ret, heap_buffer = 0;
     void *data = NULL;
 
-#if defined(WIN32) || defined(_MSC_VER)
-    if (!initWSock())
+    if (s->max_msg_size > 4096) {
+        buffer = (char*) malloc(s->max_msg_size);
+        heap_buffer = 1;
+    }
+    else if (buffer > 0) {
+        buffer = (char*) alloca(s->max_msg_size);
+    }
+
+    if (!buffer)
         return NULL;
+
+    if (s->max_msg_size<=0) {
+        if (heap_buffer) free(buffer);
+        return NULL;
+    }
+
+#if defined(WIN32) || defined(_MSC_VER)
+    if (!initWSock()) {
+        if (heap_buffer) free(buffer);
+        return NULL;
+    }
 #endif
 
     s->addr_len = sizeof(s->addr);
 
-    ret = recvfrom(s->sockets[0].fd, buffer, LO_MAX_MSG_SIZE, 0,
+    ret = recvfrom(s->sockets[0].fd, buffer, s->max_msg_size, 0,
                    (struct sockaddr *) &s->addr, &s->addr_len);
     if (ret <= 0) {
+        if (heap_buffer) free(buffer);
         return NULL;
     }
     data = malloc(ret);
@@ -811,6 +849,7 @@ void *lo_server_recv_raw(lo_server s, size_t * size)
     if (size)
         *size = ret;
 
+    if (heap_buffer) free(buffer);
     return data;
 }
 
@@ -839,6 +878,7 @@ static int slip_decode(unsigned char **buffer, unsigned char *from,
                 return 0;
             case SLIP_ESC:
                 *state = 1;
+                from++;
                 continue;
             default:
                 *(*buffer)++ = *from++;
@@ -849,9 +889,11 @@ static int slip_decode(unsigned char **buffer, unsigned char *from,
             switch (*from) {
             case SLIP_ESC_END:
                 *(*buffer)++ = SLIP_END;
+                from++;
                 break;
             case SLIP_ESC_ESC:
                 *(*buffer)++ = SLIP_ESC;
+                from++;
                 break;
             }
             *state = 0;
@@ -952,8 +994,8 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
     struct socket_context *sc = &s->contexts[isock];
     char *stack_buffer = 0, *read_into;
     uint32_t msg_len;
-	int buffer_bytes_left, size, bytes_recv;
-	size_t bytes_wrote;
+	int buffer_bytes_left, bytes_recv;
+	ssize_t bytes_wrote, size;
     *pdata = 0;
 
   again:
@@ -978,16 +1020,18 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
         // size as in UDP, however we leave it for security
         // reasons--an unterminated SLIP stream would consume memory
         // indefinitely.
-        if (size > LO_MAX_MSG_SIZE)
-            size = LO_MAX_MSG_SIZE;
+        if (s->max_msg_size != -1 && size > s->max_msg_size) {
+            size = s->max_msg_size;
+            break;
+        }
 
         buffer_bytes_left = size - sc->buffer_read_offset;
     }
 
-    if (size > sc->buffer_size)
+    if ((size_t)size > sc->buffer_size)
     {
         sc->buffer_size = size;
-        sc->buffer = realloc(sc->buffer, sc->buffer_size);
+        sc->buffer = (char*) realloc(sc->buffer, sc->buffer_size);
         if (!sc->buffer)
             // Out of memory
             return 0;
@@ -1001,7 +1045,7 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
     // In SLIP mode, we instead read into the local stack buffer
     if (sc->is_slip == 1)
     {
-        stack_buffer = alloca(buffer_bytes_left - sizeof(uint32_t));
+        stack_buffer = (char*) alloca(buffer_bytes_left - sizeof(uint32_t));
         read_into = stack_buffer;
     }
 
@@ -1030,7 +1074,7 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
         // Copy to stack if we just discovered we are in SLIP mode
         if (sc->is_slip)
         {
-            stack_buffer = alloca(bytes_recv);
+            stack_buffer = (char*) alloca(bytes_recv);
             memcpy(stack_buffer, read_into, bytes_recv);
 
             // Make room for size header
@@ -1088,7 +1132,7 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
             if (bytes_recv + sizeof(uint32_t) > sc->buffer_size - sc->buffer_read_offset)
             {
                 sc->buffer_size *= 2;
-                sc->buffer = realloc(sc->buffer, sc->buffer_size);
+                sc->buffer = (char*)  realloc(sc->buffer, sc->buffer_size);
             }
         }
 
@@ -1156,7 +1200,7 @@ void *lo_server_recv_raw_stream(lo_server s, size_t * size, int *psock)
 
     poll(s->sockets, s->sockets_len, -1);
 
-    for (i = (s->sockets_len - 1) && !data; i >= 0; --i) {
+    for (i = data ? 0 : (s->sockets_len - 1); i >= 0; --i) {
         if (s->sockets[i].revents == POLLERR
             || s->sockets[i].revents == POLLHUP) {
             if (i > 0) {
@@ -1527,7 +1571,7 @@ int lo_server_add_socket(lo_server s, int socket, lo_address a,
                            sizeof(*(s->sockets)) * (s->sockets_alloc * 2));
         if (!sp)
             return -1;
-        s->sockets = sp;
+        s->sockets = (lo_server_fd_type *) sp;
         memset((char*)sp + s->sockets_alloc*sizeof(*s->sockets),
                0, s->sockets_alloc*sizeof(*s->sockets));
 
@@ -1536,7 +1580,7 @@ int lo_server_add_socket(lo_server s, int socket, lo_address a,
                      * (s->sockets_alloc * 2));
         if (!sc)
             return -1;
-        s->contexts = sc;
+        s->contexts = (struct socket_context *) sc;
         memset((char*)sc + s->sockets_alloc*sizeof(*s->contexts),
                0, s->sockets_alloc*sizeof(*s->contexts));
 
@@ -1549,8 +1593,9 @@ int lo_server_add_socket(lo_server s, int socket, lo_address a,
     /* Update socket-indexed array of sources */
     if (socket >= s->sources_len) {
         int L = socket * 2;
-        s->sources = realloc(s->sources,
-                             sizeof(struct _lo_address) * L);
+        s->sources = (lo_address)
+            realloc(s->sources,
+                    sizeof(struct _lo_address) * L);
         memset(s->sources + s->sources_len, 0,
                sizeof(struct _lo_address) * (L - s->sources_len));
         s->sources_len = L;
@@ -1591,14 +1636,14 @@ static int dispatch_data(lo_server s, void *data,
                          size_t size, int sock)
 {
     int result = 0;
-    char *path = data;
+    char *path = (char*) data;
     ssize_t len = lo_validate_string(data, size);
     if (len < 0) {
         lo_throw(s, -len, "Invalid message path", NULL);
         return len;
     }
 
-    if (!strcmp(data, "#bundle")) {
+    if (!strcmp((const char *) data, "#bundle")) {
         char *pos;
         int remain;
         uint32_t elem_len;
@@ -1609,7 +1654,7 @@ static int dispatch_data(lo_server s, void *data,
             lo_throw(s, -bundle_result, "Invalid bundle", NULL);
             return bundle_result;
         }
-        pos = (char *) data + len;
+        pos = (char*) data + len;
         remain = size - len;
 
         lo_timetag_now(&now);
@@ -1671,7 +1716,7 @@ static int dispatch_data(lo_server s, void *data,
             return -result;
         }
         lo_message_incref(msg);
-        dispatch_method(s, data, msg, sock);
+        dispatch_method(s, (const char *)data, msg, sock);
         lo_message_free(msg);
     }
     return size;
@@ -1757,24 +1802,24 @@ static void dispatch_method(lo_server s, const char *path,
                 if (argc > 0) {
                     int i;
                     int opsize = 0;
-                    char *ptr = msg->data, *data_co_ptr = NULL;
+                    char *ptr = (char*) msg->data, *data_co_ptr = NULL;
 
-                    argv = calloc(argc, sizeof(lo_arg *));
+                    argv = (lo_arg **) calloc(argc, sizeof(lo_arg *));
                     for (i = 0; i < argc; i++) {
-                        opsize += lo_arg_size(it->typespec[i], ptr);
-                        ptr += lo_arg_size(types[i], ptr);
+                        opsize += lo_arg_size((lo_type)it->typespec[i], ptr);
+                        ptr += lo_arg_size((lo_type)types[i], ptr);
                     }
 
-                    data_co = malloc(opsize);
+                    data_co = (char*) malloc(opsize);
                     data_co_ptr = data_co;
-                    ptr = msg->data;
+                    ptr = (char*) msg->data;
                     for (i = 0; i < argc; i++) {
                         argv[i] = (lo_arg *) data_co_ptr;
-                        lo_coerce(it->typespec[i], (lo_arg *) data_co_ptr,
-                                  types[i], (lo_arg *) ptr);
+                        lo_coerce((lo_type)it->typespec[i], (lo_arg *) data_co_ptr,
+                                  (lo_type)types[i], (lo_arg *) ptr);
                         data_co_ptr +=
-                        lo_arg_size(it->typespec[i], data_co_ptr);
-                        ptr += lo_arg_size(types[i], ptr);
+                            lo_arg_size((lo_type)it->typespec[i], data_co_ptr);
+                        ptr += lo_arg_size((lo_type)types[i], ptr);
                     }
                 }
 
@@ -1799,7 +1844,7 @@ static void dispatch_method(lo_server s, const char *path,
 
     /* If we find no matching methods, check for protocol level stuff */
     if (ret == 1 && s->protocol == LO_UDP) {
-        char *pos = strrchr(path, '/');
+        char *pos = (char*) strrchr(path, '/');
 
         /* if its a method enumeration call */
         if (pos && *(pos + 1) == '\0') {
@@ -1819,7 +1864,7 @@ static void dispatch_method(lo_server s, const char *path,
                     char *tmp;
                     char *sec;
 
-                    tmp = malloc(strlen(it->path + len) + 1);
+                    tmp = (char*) malloc(strlen(it->path + len) + 1);
                     strcpy(tmp, it->path + len);
 #if defined(WIN32) || defined(_MSC_VER)
                     sec = strchr(tmp, '/');
@@ -1837,7 +1882,7 @@ static void dispatch_method(lo_server s, const char *path,
                         }
                     }
                     if (tmp) {
-                        slnew = calloc(1, sizeof(lo_strlist));
+                        slnew = (lo_strlist *) calloc(1, sizeof(lo_strlist));
                         slnew->str = tmp;
                         slnew->next = NULL;
                         if (!slend) {
@@ -1875,9 +1920,10 @@ static void queue_data(lo_server s, lo_timetag ts, const char *path,
                        lo_message msg, int sock)
 {
     /* insert blob into future dispatch queue */
-    queued_msg_list *it = s->queued;
+    queued_msg_list *it = (queued_msg_list *) s->queued;
     queued_msg_list *prev = NULL;
-    queued_msg_list *ins = calloc(1, sizeof(queued_msg_list));
+    queued_msg_list *ins = (queued_msg_list *)
+        calloc(1, sizeof(queued_msg_list));
 
     ins->ts = ts;
     ins->path = strdup(path);
@@ -1897,7 +1943,7 @@ static void queue_data(lo_server s, lo_timetag ts, const char *path,
             return;
         }
         prev = it;
-        it = it->next;
+        it = (queued_msg_list *) it->next;
     }
 
     /* fell through, so this event is last */
@@ -1911,7 +1957,7 @@ static void queue_data(lo_server s, lo_timetag ts, const char *path,
 
 static int dispatch_queued(lo_server s, int dispatch_all)
 {
-    queued_msg_list *head = s->queued;
+    queued_msg_list *head = (queued_msg_list *) s->queued;
     queued_msg_list *tailhead;
     lo_timetag disp_time;
 
@@ -1927,7 +1973,7 @@ static int dispatch_queued(lo_server s, int dispatch_all)
         char *path;
         lo_message msg;
         int sock;
-        tailhead = head->next;
+        tailhead = (queued_msg_list *) head->next;
         path = ((queued_msg_list *) s->queued)->path;
         msg = ((queued_msg_list *) s->queued)->msg;
         sock = ((queued_msg_list *) s->queued)->sock;
@@ -1946,9 +1992,9 @@ static int dispatch_queued(lo_server s, int dispatch_all)
 
 lo_method lo_server_add_method(lo_server s, const char *path,
                                const char *typespec, lo_method_handler h,
-                               void *user_data)
+                               const void *user_data)
 {
-    lo_method m = calloc(1, sizeof(struct _lo_method));
+    lo_method m = (lo_method) calloc(1, sizeof(struct _lo_method));
     lo_method it;
 
     if (path && strpbrk(path, " #*,?[]{}")) {
@@ -1971,7 +2017,7 @@ lo_method lo_server_add_method(lo_server s, const char *path,
     }
 
     m->handler = h;
-    m->user_data = user_data;
+    m->user_data = (char*) user_data;
     m->next = NULL;
 
     /* append the new method to the list */
@@ -2031,6 +2077,40 @@ void lo_server_del_method(lo_server s, const char *path,
     }
 }
 
+int lo_server_del_lo_method(lo_server s, lo_method m)
+{
+    lo_method it, prev, next;
+
+    if (!s->first)
+        return 1;
+
+    it = s->first;
+    prev = it;
+    while (it) {
+        /* incase we free it */
+        next = it->next;
+
+        if (it == m) {
+            /* Take care when removing the head. */
+            if (it == s->first) {
+                s->first = it->next;
+            } else {
+                prev->next = it->next;
+            }
+            next = it->next;
+            free((void *) it->path);
+            free((void *) it->typespec);
+            free(it);
+            it = prev;
+            return 0;
+        }
+        prev = it;
+        if (it)
+            it = next;
+    }
+    return 1;
+}
+
 int lo_server_add_bundle_handlers(lo_server s,
                                   lo_bundle_start_handler sh,
                                   lo_bundle_end_handler eh,
@@ -2085,6 +2165,9 @@ char *lo_server_get_url(lo_server s)
     if (s->protocol == LO_UDP || s->protocol == LO_TCP) {
         const char *proto = s->protocol == LO_UDP ? "udp" : "tcp";
 
+        if (!s->hostname)
+            lo_server_resolve_hostname(s);
+
 #ifndef _MSC_VER
         ret =
             snprintf(NULL, 0, "osc.%s://%s:%d/", proto, s->hostname,
@@ -2094,7 +2177,7 @@ char *lo_server_get_url(lo_server s)
             /* this libc is not C99 compliant, guess a size */
             ret = 1023;
         }
-        buf = malloc((ret + 2) * sizeof(char));
+        buf = (char*) malloc((ret + 2) * sizeof(char));
         snprintf(buf, ret + 1, "osc.%s://%s:%d/", proto, s->hostname,
                  s->port);
 
@@ -2107,7 +2190,7 @@ char *lo_server_get_url(lo_server s)
             /* this libc is not C99 compliant, guess a size */
             ret = 1023;
         }
-        buf = malloc((ret + 2) * sizeof(char));
+        buf = (char*) malloc((ret + 2) * sizeof(char));
         snprintf(buf, ret + 1, "osc.unix:///%s", s->path);
 
         return buf;
@@ -2147,29 +2230,50 @@ static int lo_can_coerce_spec(const char *a, const char *b)
 
 static int lo_can_coerce(char a, char b)
 {
-    return ((a == b) ||
-            (lo_is_numerical_type(a) && lo_is_numerical_type(b)) ||
-            (lo_is_string_type(a) && lo_is_string_type(b)));
+    return
+        ((a == b) ||
+         (lo_is_numerical_type((lo_type) a) &&
+          lo_is_numerical_type((lo_type) b)) ||
+         (lo_is_string_type((lo_type) a) &&
+          lo_is_string_type((lo_type) b)));
 }
 
 // Context for error handler
 void *lo_error_context;
 #ifdef ENABLE_THREADS
-pthread_mutex_t lo_error_context_mutex = PTHREAD_MUTEX_INITIALIZER;
+#ifdef HAVE_LIBPTHREAD
+    pthread_mutex_t lo_error_context_mutex = PTHREAD_MUTEX_INITIALIZER;
+#else
+#ifdef HAVE_WIN32_THREADS
+    CRITICAL_SECTION lo_error_context_mutex = {(void*)-1,-1,0,0,0,0};
+#endif
+#endif
 #endif
 
 void lo_throw(lo_server s, int errnum, const char *message,
               const char *path)
 {
     if (s->err_h) {
-        #ifdef ENABLE_THREADS
+#ifdef ENABLE_THREADS
+#ifdef HAVE_LIBPTHREAD
         pthread_mutex_lock(&lo_error_context_mutex);
-        #endif
+#else
+#ifdef HAVE_WIN32_THREADS
+        EnterCriticalSection (&lo_error_context_mutex);
+#endif
+#endif
+#endif
         lo_error_context = s->error_user_data;
         (*s->err_h) (errnum, message, path);
-        #ifdef ENABLE_THREADS
-        pthread_mutex_unlock(&lo_error_context_mutex);
-        #endif
+#ifdef ENABLE_THREADS
+#ifdef HAVE_LIBPTHREAD
+        pthread_mutex_unlock (&lo_error_context_mutex);
+#else
+#ifdef HAVE_WIN32_THREADS
+        LeaveCriticalSection (&lo_error_context_mutex);
+#endif
+#endif
+#endif
     }
 }
 
@@ -2183,4 +2287,29 @@ void lo_server_set_error_context(lo_server s, void *user_data)
     s->error_user_data = user_data;
 }
 
+int lo_server_max_msg_size(lo_server s, int req_size)
+{
+    if (req_size == 0)
+        return s->max_msg_size;
+
+    if (s->protocol == LO_UDP) {
+        if (req_size > 65535)
+            req_size = 65535;
+
+        if (req_size < 0)
+            return s->max_msg_size;
+    }
+    else if (s->protocol == LO_TCP)
+    {
+        // TODO: We could potentially shrink the TCP buffers here
+        // if req_size < buffer_size for any open sockets, but
+        // care must be taken to clean up any data already in the
+        // buffers.
+    }
+
+    s->max_msg_size = req_size;
+
+    return s->max_msg_size;
+}
+
 /* vi:set ts=8 sts=4 sw=4: */
